Class: Redwood::Chunk::Attachment

Inherits:
Object
  • Object
show all
Defined in:
lib/sup/message_chunks.rb

Constant Summary collapse

@@view_tempfiles =

store tempfile objects as class variables so that they are not removed when the viewing process returns. they should be garbage collected when the class variable is removed.

[]

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(content_type, filename, encoded_content, sibling_types) ⇒ Attachment

Returns a new instance of Attachment.



88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
# File 'lib/sup/message_chunks.rb', line 88

def initialize content_type, filename, encoded_content, sibling_types
  @content_type = content_type.downcase
  if Shellwords.escape(@content_type) != @content_type
    warn "content_type #{@content_type} is not safe, changed to application/octet-stream"
    @content_type = 'application/octet-stream'
  end

  @filename = filename
  @quotable = false # changed to true if we can parse it through the
                    # mime-decode hook, or if it's plain text
  @raw_content =
    if encoded_content.body
      encoded_content.decode
    else
      "For some bizarre reason, RubyMail was unable to parse this attachment.\n"
    end

  text = case @content_type
  when /^text\/plain\b/
    if /^UTF-7$/i =~ encoded_content.charset
      @raw_content.decode_utf7
    else
      begin
        charset = Encoding.find(encoded_content.charset || 'US-ASCII')
      rescue ArgumentError
        charset = 'US-ASCII'
      end
      @raw_content.force_encoding(charset)
    end
  else
    HookManager.run "mime-decode", :content_type => @content_type,
                    :filename => lambda { write_to_disk },
                    :charset => encoded_content.charset,
                    :sibling_types => sibling_types
  end

  @lines = nil
  if text
    text = text.encode($encoding, :invalid => :replace, :undef => :replace)
    begin
      @lines = text.gsub("\r\n", "\n").gsub(/\t/, "        ").gsub(/\r/, "").split("\n")
    rescue Encoding::CompatibilityError
      @lines = text.fix_encoding!.gsub("\r\n", "\n").gsub(/\t/, "        ").gsub(/\r/, "").split("\n")
      debug "error while decoding message text, falling back to default encoding, expect errors in encoding: #{text.fix_encoding!}"
    end

    @quotable = true
  end
end

Instance Attribute Details

#content_typeObject (readonly)

raw_content is the post-MIME-decode content. this is used for saving the attachment to disk.



80
81
82
# File 'lib/sup/message_chunks.rb', line 80

def content_type
  @content_type
end

#filenameObject (readonly)

raw_content is the post-MIME-decode content. this is used for saving the attachment to disk.



80
81
82
# File 'lib/sup/message_chunks.rb', line 80

def filename
  @filename
end

#linesObject (readonly)

raw_content is the post-MIME-decode content. this is used for saving the attachment to disk.



80
81
82
# File 'lib/sup/message_chunks.rb', line 80

def lines
  @lines
end

#raw_contentObject (readonly)

raw_content is the post-MIME-decode content. this is used for saving the attachment to disk.



80
81
82
# File 'lib/sup/message_chunks.rb', line 80

def raw_content
  @raw_content
end

Instance Method Details

#colorObject



138
# File 'lib/sup/message_chunks.rb', line 138

def color; :text_color end

#expandable?Boolean

Returns:

  • (Boolean)


153
# File 'lib/sup/message_chunks.rb', line 153

def expandable?; !viewable? end

#filesafe_filenameObject



148
# File 'lib/sup/message_chunks.rb', line 148

def filesafe_filename; @filename.gsub("/", "_") end

#indexable?Boolean

Returns:

  • (Boolean)


154
# File 'lib/sup/message_chunks.rb', line 154

def indexable?; expandable? end

#initial_stateObject



155
# File 'lib/sup/message_chunks.rb', line 155

def initial_state; :open end

#inlineable?Boolean

an attachment is exapndable if we’ve managed to decode it into something we can display inline. otherwise, it’s viewable.

Returns:

  • (Boolean)


152
# File 'lib/sup/message_chunks.rb', line 152

def inlineable?; false end

#patina_colorObject



139
# File 'lib/sup/message_chunks.rb', line 139

def patina_color; :attachment_color end

#patina_textObject



140
141
142
143
144
145
146
# File 'lib/sup/message_chunks.rb', line 140

def patina_text
  if expandable?
    "Attachment: #{filename} (#{lines.length} lines)"
  else
    "Attachment: #{filename} (#{content_type}; #{@raw_content.size.to_human_size})"
  end
end

#safe_filenameObject



147
# File 'lib/sup/message_chunks.rb', line 147

def safe_filename; Shellwords.escape(@filename).gsub("/", "_") end

#to_sObject

used when viewing the attachment as text



203
204
205
# File 'lib/sup/message_chunks.rb', line 203

def to_s
  @lines || @raw_content
end

#view!Object



169
170
171
172
173
174
175
# File 'lib/sup/message_chunks.rb', line 169

def view!
  write_to_disk do |path|
    ret = HookManager.run "mime-view", :content_type => @content_type,
                                       :filename => path
    ret || view_default!(path)
  end
end

#view_default!(path) ⇒ Object



157
158
159
160
161
162
163
164
165
166
167
# File 'lib/sup/message_chunks.rb', line 157

def view_default! path
  case RbConfig::CONFIG['arch']
    when /darwin/
      cmd = "open #{path}"
    else
      cmd = "/usr/bin/run-mailcap --action=view #{@content_type}:#{path}"
  end
  debug "running: #{cmd.inspect}"
  BufferManager.shell_out(cmd)
  $? == 0
end

#viewable?Boolean

Returns:

  • (Boolean)


156
# File 'lib/sup/message_chunks.rb', line 156

def viewable?; @lines.nil? end

#write_to_diskObject



177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
# File 'lib/sup/message_chunks.rb', line 177

def write_to_disk
  begin
    # Add the original extension to the generated tempfile name only if the
    # extension is "safe" (won't be interpreted by the shell).  Since
    # Tempfile.new always generates safe file names this should prevent
    # attacking the user with funny attachment file names.
    tempname = if (File.extname @filename) =~ /^\.[[:alnum:]]+$/ then
                 ["sup-attachment", File.extname(@filename)]
               else
                 "sup-attachment"
               end

    file = Tempfile.new(tempname)
    file.print @raw_content
    file.flush

    @@view_tempfiles.push file # make sure the tempfile is not garbage collected before sup stops

    yield file.path if block_given?
    return file.path
  ensure
    file.close
  end
end